import _extends from "@babel/runtime/helpers/extends";
import _inheritsLoose from "@babel/runtime/helpers/inheritsLoose";
import * as THREE from 'three';
import Object3 from 'awv3/three/object3';
import { createCaption } from 'awv3/three/helpers';

var Axes =
/*#__PURE__*/
function (_THREE$Object3D) {
  _inheritsLoose(Axes, _THREE$Object3D);

  function Axes(scene, camera, cubeProps) {
    var _this;

    _this = _THREE$Object3D.call(this) || this;
    var xAxisColor = 'rgb(194, 51, 105)'; //0xc23369

    var yAxisColor = 'rgb(51, 194, 105)'; //0x33c269

    var zAxisColor = 'rgb(51, 105, 194)'; //0x3369c2

    var radius = cubeProps.radius;
    _this.camera = camera;
    var cylinderRadius = 0.84;
    var cylinderLength = 2 * radius;

    _this.viewFound().then(function (view) {
      if (_this.destroyed) return;

      _this.createInteraction({
        priority: 1000
      }).on(Object3.Events.Lifecycle.Rendered, _this.onRender, {
        sync: false
      });
    });

    _this.xAxisEndPos = new THREE.Vector3(0, cylinderLength / 2, 0);
    _this.yAxisEndPos = new THREE.Vector3(0, cylinderLength / 2, 0);
    _this.zAxisEndPos = new THREE.Vector3(0, cylinderLength / 2, 0);
    var material = new THREE.MeshBasicMaterial({
      opacity: 1,
      side: THREE.DoubleSide,
      color: new THREE.Color(xAxisColor)
    });
    var cylinderGeometry = new THREE.CylinderGeometry(cylinderRadius, cylinderRadius, cylinderLength);
    var axis = new THREE.Mesh(cylinderGeometry, material);
    axis.position.set(0, -radius, -radius);
    axis.quaternion.setFromUnitVectors(new THREE.Vector3(0, 1, 0), new THREE.Vector3(1, 0, 0));
    axis.updateMatrix();

    _this.xAxisEndPos.applyMatrix4(axis.matrix);

    _this.add(axis);

    var label = createCaption('X', xAxisColor);
    label.position.set(_this.xAxisEndPos.x, _this.xAxisEndPos.y, _this.xAxisEndPos.z);

    _this.add(label);

    material = new THREE.MeshBasicMaterial({
      opacity: 1,
      side: THREE.DoubleSide,
      color: new THREE.Color(yAxisColor)
    });
    axis = new THREE.Mesh(cylinderGeometry, material);
    axis.position.set(-radius, 0, -radius);
    axis.quaternion.setFromUnitVectors(new THREE.Vector3(0, 1, 0), new THREE.Vector3(0, 1, 0));
    axis.updateMatrix();

    _this.yAxisEndPos.applyMatrix4(axis.matrix);

    _this.add(axis);

    label = createCaption('Y', yAxisColor);
    label.position.set(_this.yAxisEndPos.x, _this.yAxisEndPos.y, _this.yAxisEndPos.z);

    _this.add(label);

    material = new THREE.MeshBasicMaterial({
      opacity: 1,
      side: THREE.DoubleSide,
      color: new THREE.Color(zAxisColor)
    });
    axis = new THREE.Mesh(cylinderGeometry, material);
    axis.position.set(-radius, -radius, 0);
    axis.quaternion.setFromUnitVectors(new THREE.Vector3(0, 1, 0), new THREE.Vector3(0, 0, 1));
    axis.updateMatrix();

    _this.zAxisEndPos.applyMatrix4(axis.matrix);

    _this.add(axis);

    label = createCaption('Z', zAxisColor);
    label.position.set(_this.zAxisEndPos.x, _this.zAxisEndPos.y, _this.zAxisEndPos.z);

    _this.add(label);

    return _this;
  }

  return Axes;
}(THREE.Object3D);

var ViewCube =
/*#__PURE__*/
function () {
  function ViewCube() {}

  ViewCube.init = function init(props) {
    var radius = props.radius,
        chamfer = props.chamfer,
        opacity = props.opacity,
        viewSession = props.viewSession,
        viewCsys = props.viewCsys,
        showAxes = props.showAxes,
        showFaceNames = props.showFaceNames;
    var viewCubeFaces = new THREE.Object3D();
    var shader = THREE.MeshBasicMaterial;
    var shaderProps = {
      color: 0xffffff,
      polygonOffset: true,
      polygonOffsetFactor: 1.0,
      polygonOffsetUnits: 5.0,
      transparent: opacity < 1,
      opacity: opacity
    };
    var edgeNames = ['LEFT', 'DOWN', 'BACK', 'FRONT', 'UP', 'RIGHT'];
    var fontSizePx = 15;
    var count = 0;
    var faceNum = 0;

    for (var sx = -1; sx <= 1; sx++) {
      for (var sy = -1; sy <= 1; sy++) {
        var _loop = function _loop() {
          var _face$createInteracti;

          cntNnz = +(sx != 0) + (sy != 0) + (sz != 0);
          if (cntNnz === 0) return "continue";
          var geometry = void 0,
              material = void 0;
          var ctr = void 0,
              normal = void 0,
              axisX = void 0,
              axisY = void 0;

          if (cntNnz === 1) {
            //create a square face
            size = 2 * radius * (1 - chamfer);
            geometry = new THREE.PlaneGeometry(size, size); //set material (load texture)

            material = new shader(_extends({}, shaderProps));

            if (showFaceNames) {
              var canvas = document.createElement('canvas');
              canvas.width = canvas.height = 64; // should be always a power of two (2^x) (WebGL performance)

              var context = canvas.getContext('2d');
              context.fillStyle = 'white';
              context.fillRect(0, 0, canvas.width, canvas.height);
              context.font = "Bold " + fontSizePx + "px Arial";
              context.fillStyle = 'black';
              var w = context.measureText(edgeNames[faceNum]).width;
              var h = fontSizePx;
              context.fillText(edgeNames[faceNum], (canvas.width - w) / 2, (canvas.height + 0.8 * h) / 2);
              var texture = new THREE.Texture(canvas);
              texture.needsUpdate = true;
              material.map = texture;
            }

            if (props.textures) {
              new THREE.TextureLoader().load(props.textures[count++], function (texture) {
                material.map = texture;
                material.needsUpdate = true;
              });
            } //choose coordinate system for the face


            normal = new THREE.Vector3(sx, sy, sz);
            axisX = sy === 0 ? new THREE.Vector3(sz, 0, -sx) : new THREE.Vector3(1, 0, 0);
            axisY = normal.clone().cross(axisX);
            ctr = normal.clone().multiplyScalar(radius);
            ++faceNum;
          }

          if (cntNnz === 2) {
            //calculate endpoints of the edge being chamfered
            pM = new THREE.Vector3(sx, sy, sz);
            dir = new THREE.Vector3(+(sx == 0), +(sy == 0), +(sz == 0));
            pE = pM.clone().sub(dir);
            pS = pM.clone().add(dir); //create a rectangular face

            len = 2 * radius * (1 - chamfer);
            wid = radius * chamfer * Math.sqrt(2.0);
            geometry = new THREE.PlaneGeometry(len, wid);
            material = new shader(_extends({}, shaderProps)); //calculate coordinate system

            normal = pM.clone().normalize();
            axisX = dir.clone();
            axisY = normal.clone().cross(axisX);
            ctr = pM.clone().multiplyScalar(radius * (1 - chamfer / 2));
          }

          if (cntNnz === 3) {
            //create triangle geometry for vertex chamfer
            side = radius * chamfer * Math.sqrt(2.0);
            geometry = new THREE.Geometry();
            k = 1 - chamfer;
            geometry.vertices.push(new THREE.Vector3(-0.5, Math.sqrt(1 / 3) / 2, 0).multiplyScalar(side), new THREE.Vector3(0.5, Math.sqrt(1 / 3) / 2, 0).multiplyScalar(side), new THREE.Vector3(0, -Math.sqrt(1 / 3), 0).multiplyScalar(side));
            geometry.faces.push(new THREE.Face3(0, 1, 2));
            if (sy > 0) geometry.scale(-1, -1, -1);
            geometry.computeBoundingSphere();
            material = new shader(_extends({}, shaderProps, {
              side: THREE.DoubleSide
            })); //calculate coordinate system

            sp = new THREE.Vector3(sx, sy, sz);
            normal = sp.clone().normalize();
            axisY = new THREE.Vector3(0, 1, 0);
            axisY.sub(axisY.clone().projectOnVector(normal)).normalize();
            axisX = axisY.clone().cross(normal);
            ctr = sp.clone().multiplyScalar(radius * (1 - chamfer * 2 / 3));
          }

          geometry.computeFaceNormals();
          face = new THREE.Mesh(geometry, material); //set coordinate system to THREE object

          matr = new THREE.Matrix4();
          matr.makeBasis(axisX, axisY, normal);
          matr.setPosition(ctr);
          matr.decompose(face.position, face.quaternion, face.scale);
          face.updateMatrix(); //add object

          viewCubeFaces.add(face); //add interaction

          var a2 = Math.acos(normal.y);
          var a1 = Math.abs(normal.y) < 1 ? Math.atan2(normal.x, normal.z) : 0;
          face.createInteraction().on((_face$createInteracti = {}, _face$createInteracti[Object3.Events.Interaction.Hovered] = function (data) {
            viewCsys.setCursor('pointer');
            data.material.animate({
              color: new THREE.Color(0xffffff - 0x101010)
            }).start(0);
          }, _face$createInteracti[Object3.Events.Interaction.Unhovered] = function (data) {
            data.material.animate({
              color: new THREE.Color(0xffffff)
            }).start(1000);
          }, _face$createInteracti[Object3.Events.Interaction.Clicked] = function (clickedObj) {
            viewSession.controls.rotate(a1, a2); // Works properly only if camera.up === (0,0,1)
          }, _face$createInteracti));
        };

        for (var sz = -1; sz <= 1; sz++) {
          var cntNnz;
          var size;
          var pM;
          var dir;
          var pE;
          var pS;
          var len;
          var wid;
          var side;
          var k;
          var sp;
          var face;
          var matr;

          var _ret = _loop();

          if (_ret === "continue") continue;
        }
      }
    } //add wireframe geometry


    var viewCubeEdges = new THREE.Object3D();

    for (var _iterator = viewCubeFaces.children, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
      var _ref;

      if (_isArray) {
        if (_i >= _iterator.length) break;
        _ref = _iterator[_i++];
      } else {
        _i = _iterator.next();
        if (_i.done) break;
        _ref = _i.value;
      }

      var face = _ref;
      var pts = face.geometry.vertices.slice();
      if (face.geometry.faces.length === 2) pts = [pts[0], pts[1], pts[3], pts[2]];
      pts.push(pts[0].clone());
      var geom = new THREE.Geometry();
      geom.vertices = pts;
      var lines = new THREE.Line(geom, new THREE.LineBasicMaterial({
        color: 0,
        transparent: true,
        opacity: 0.3
      }));
      lines.renderOrder = -1000;
      face.matrix.decompose(lines.position, lines.quaternion, lines.scale);
      lines.updateMatrix();
      viewCubeEdges.add(lines);
    } // Works properly only if camera.up === (0,0,1)


    viewCubeFaces.quaternion.setFromUnitVectors(viewCubeFaces.up, viewCsys.camera.up);
    var target = viewCsys.scene;
    target.add(viewCubeFaces);
    target.add(viewCubeEdges);
    target.update && target.update();
    viewCsys.controls.noZoom = viewCsys.controls.noRotate = viewCsys.controls.noPan = true;

    viewSession.callbackAfter = function () {
      var src = viewSession.camera;
      var dst = viewCsys.camera;
      var oldAspect = dst.aspect;
      dst.copy(src, false);

      if (oldAspect !== undefined) {
        dst.aspect = oldAspect;
        dst.updateProjectionMatrix();
      }

      var worldDir = new THREE.Vector3();
      src.getWorldDirection(worldDir);
      dst.position.copy(worldDir.clone().multiplyScalar(-100.0));
      viewCsys.invalidate();
    };

    if (showAxes) {
      var axes = new Axes(target, viewCsys.camera, props);
      target.add(axes);
    }
  };

  return ViewCube;
}();

export { ViewCube as default };